clear
close all

% Nonlinear_feature_extraction

%% datasets
dataset =1;    % 1 - colon cancer Alon PNAS 1999; 
               % 2 - diffusive B cell lymphoma (DBCL), Nature Medicine 2002
               % 3 - prostate cancer
               % 4 - MS ovarian cancer Petricoin et al. The Lancet 2002 
               % 5 - MS prostate cancer Petricoin et al. J. Nat Canc Inst 2002

%% mapping parameters
                     
d = 5;               % order of approximate EFM
sig2 = 1.0e0;        % variance of Gaussian EFM

%% sparseness regularization constant
lambda = 0.1;

%% processing
if dataset == 1
    cd .\Colon_cancer
    load gene_profiles_colon_ordered.txt
    [T N] = size(gene_profiles_colon_ordered);
    X = gene_profiles_colon_ordered;
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:22
        control_ref = control_ref + (1/22)*X(:,i)';   % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:40
        cancer_ref = cancer_ref + (1/40)*X(:,i+22)';  % Cancer
    end
elseif dataset == 2
    cd .\Diffuse_B_cell_lymphoma
    load DLBCL
    [N T] = size(data);
    X = data(:,2:T)';
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:58
        control_ref = control_ref + (1/58)*X(:,i)';     % DLBCL
    end
    cancer_ref = zeros(1,p);
    for i=1:19
        cancer_ref = cancer_ref + (1/19)*X(:,i+58)';    % FL
    end
elseif dataset == 3
    cd .\Prostate_cancer_gene_profiles
    load Prostate_Tumor
    [N T] = size(data);
    X = data(:,2:T)';
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:50
        control_ref = control_ref + (1/50)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:52
        cancer_ref = cancer_ref + (1/52)*X(:,i+50)';    % Cancer
    end
elseif dataset == 4
    cd .\Ovarian_cancer_MS\Ovarian_Data_WCX2_CSV\control\Group_C    % Low resolution SELDI TOF MS data
    load Control_C
    [T N] = size(Control_C);
    X = Control_C(3:T,2:51);
    cd .. 
    cd .\Group_D    % Low resolution SELDI TOF MS data
    load Control_D
    X(:,51:100) = Control_D(3:T,2:51);
    cd ..
    cd ..

    cd .\cancer\Group_A    % Low resolution SELDI TOF MS data
    load Cancer_A
    X(:,101:150) = Cancer_A(3:T,2:51);
    cd ..

    cd .\Group_B    % Low resolution SELDI TOF MS data
    load Cancer_B
    X(:,151:200) = Cancer_B(3:T,2:51);
    xm = max(max(abs(X)));
    X = X/xm;
    [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:100
        control_ref = control_ref + (1/100)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:100
        cancer_ref = cancer_ref + (1/100)*X(:,i+100)';    % Cancer
    end   
elseif dataset == 5
    cd .\Prostate_cancer_MS\JNCI_Data_7_3_02\No_Evidence_of_Disease_PSA_Less_1    % No evidence of cancer, PSA less than 1
    load Control_PSA_less_1
    [T N] = size(Control_PSA_less_1);
    X = Control_PSA_less_1(:,1:63);
    cd ..

    cd .\Prostate_Cancer_PSA_Greater_10    % Cancer, PSA greater than 10
    load Cancer_PSA_greater_10
    X(:,64:106) = Cancer_PSA_greater_10(:,1:43);
    cd ..
    
    cd .\Prostate_Cancer_PSA_4_10
    load Cancer_PSA_4_10
    X(:,107:132)=Cancer_PSA_4_10(:,1:26);
    xm = max(max(abs(X)));
    X = X/xm;
   [p N]=size(X);
    control_ref = zeros(1,p);
    for i=1:63
        control_ref = control_ref + (1/63)*X(:,i)';     % Normal
    end
    cancer_ref = zeros(1,p);
    for i=1:69
        cancer_ref = cancer_ref + (1/69)*X(:,i+63)';    % Cancer
    end   
end
 
Xtz(1,:)=control_ref;
Xtc(1,:)=cancer_ref;

if dataset == 1
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Colon_cancer\Gauss_EFM_d_5
elseif dataset == 5
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Diffuse_B_cell_lymphoma\Gauss_EFM_d_5
elseif dataset == 6
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_gene_profiles\Gauss_EFM_d_5
   % cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_gene_profiles\linear
elseif dataset == 8
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Ovarian_cancer_MS\Gauss_EFM_d_6
elseif dataset == 9
    cd D:\MATLAB\work\HrZZ_2013_2015\Bioinformatics_experiments\Prostate_cancer_MS\Gauss_EFM_d_5
end

for nz=1:N
    nz
   % *********************************************************************
    %  Reference control
    %**********************************************************************
 
    Xtz(2,:)= X(:,nz)';
    
    sig1 = sqrt(sig2);
    y = [Xtz(2,:); Xtz(1,:)];
    FIX = feature_map(y,'gaussian',sig1,d);  
    [D TL]=size(FIX);
   
    % estimation of basis matrix by clustering (Fast separable NMF
    % algorithm)
    
    [J,normM,U] = FastSepNMF(FIX,D,0);   % for determined problems
    AH=FIX(:,J);
    M = size(AH,2);
     
    er=zeros(D,1);
    er(2,1)=1;
    
    for m=1:M
        tmp(m)= AH(:,m)'*er/norm(AH(:,m));  % cosine of the angle
    end
    [angm ind_c] = min(tmp);
    [angm ind_z] = max(tmp);
    clear tmp

    f = ones(M,1);
    lb = zeros(M,1);
    ub = 1e6*ones(M,1);

       tic
      % lambda = 0.2;
       lam = lambda * ones(1, TL);
       List = 2 * eigs(AH' * AH, 1) * ones(1, TL);
       maxiter = 100;
       SH = iterative_shrinkage_l2_batch(AH, FIX, M,'lambda', lam/2, 'maxiter', maxiter,'L', List);
       toc
       if map_type == 1
           for m=1:M
               tmp(m)=SH(m,:)*FIX(2,:)'/norm(SH(m,:))/norm(FIX(2,:));
           end
        elseif map_type == 2
           for m=1:M
               tmp(m)=SH(m,:)*FIX(D-1,:)'/norm(SH(m,:))/norm(FIX(D-1,:));
           end           
       end

    Supercomp_cancer_ref_control(nz,:)=SH(ind_c,:);
    Supercomp_control_ref_control(nz,:)=SH(ind_z,:);
    clear SH
    
    % *********************************************************************
    %  Reference cancer
    %**********************************************************************
    Xtc(2,:)= X(:,nz)';
    
    if map_type == 1
        sig1 = sqrt(sig2);
        y = [Xtc(2,:); Xtc(1,:)];
        FIX = feature_map(y,'gaussian',sig1,d);  
    elseif map_type == 2
        FIX = feature_map(Xtc,'polynomial',d,b);
    end

    [D TL]=size(FIX);

    % estimation of basis matrix by clustering (Fast separable NMF
    % algorithm)
    
    [J,normM,U] = FastSepNMF(FIX,D,0);   % for determined problems
    AH=FIX(:,J);
    M = size(AH,2);
    
    if map_type == 1
        er=zeros(D,1);
        er(2,1)=1;
    elseif map_type == 2
        er=zeros(D,1);
        er(D-1,1)=1;
    end
    
    for m=1:M
        tmp(m)= AH(:,m)'*er/norm(AH(:,m));  % cosine of the angle
    end
    [angm ind_z] = min(tmp);
    [angm ind_c] = max(tmp);
    clear tmp
    
    f = ones(M,1); 
    lb = zeros(M,1);
    ub = 1e6*ones(M,1);

       lam = lambda * ones(1, TL);
       List = 2 * eigs(AH' * AH, 1) * ones(1, TL);
       maxiter = 100;
       SH = iterative_shrinkage_l2_batch(AH, FIX, M,'lambda', lam/2, 'maxiter', maxiter,'L', List);
       toc
       if map_type == 1
           for m=1:M
               tmp(m)=SH(m,:)*FIX(2,:)'/norm(SH(m,:))/norm(FIX(2,:));
           end
        elseif map_type == 2
           for m=1:M
               tmp(m)=SH(m,:)*FIX(D-1,:)'/norm(SH(m,:))/norm(FIX(D-1,:));
           end           
       end
 
     
    Supercomp_cancer_ref_cancer(nz,:)=SH(ind_c,:);
    Supercomp_control_ref_cancer(nz,:)=SH(ind_z,:);
    clear SH
end

figure(1)
title('Cancer ref. control')
for n=1:62
    subplot(8,8,n)
    plot(Supercomp_cancer_ref_control(n,:))
    set(gca,'Ytick',[]);
    set(gca,'Xtick',[]);
end

figure(2)
title('Cancer ref. cancer')
for n=1:62
    subplot(8,8,n)
    plot(Supercomp_cancer_ref_cancer(n,:))
    set(gca,'Ytick',[]);
    set(gca,'Xtick',[]);
end

figure(3)
title('Control ref. cancer')
for n=1:62
    subplot(8,8,n)
    plot(Supercomp_control_ref_cancer(n,:))
    set(gca,'Ytick',[]);
    set(gca,'Xtick',[]);
end

figure(4)
title('Control ref. control')
for n=1:62
    subplot(8,8,n)
    plot(Supercomp_control_ref_control(n,:))
    set(gca,'Ytick',[]);
    set(gca,'Xtick',[]);
end

for i=1:p
    var_gen(i)=var(Supercomp_cancer_ref_cancer(:,i));
end

figure
stem(var_gen)

[var_sort ind_sort] = sort(var_gen,'descend');

name=strcat('sorted_variance_cancer_cancer_lam_0p',num2str(10*lambda));
save(name,'var_sort')
name=strcat('sorted_indices_cancer_cancer_lam_0p',num2str(10*lambda));
save(name,'ind_sort')

for i=1:p
    var_gen(i)=var(Supercomp_control_ref_cancer(:,i));
end

figure
stem(var_gen)

[var_sort ind_sort] = sort(var_gen,'descend');

name=strcat('sorted_variance_control_cancer_lam_0p',num2str(10*lambda));
save(name,'var_sort')
name=strcat('sorted_indices_control_cancer_lam_0p',num2str(10*lambda));
save(name,'ind_sort')

for i=1:p
    var_gen(i)=var(Supercomp_control_ref_control(:,i));
end

figure
stem(var_gen)

[var_sort ind_sort] = sort(var_gen,'descend');

name=strcat('sorted_variance_control_control_lam_0p',num2str(10*lambda));
save(name,'var_sort')
name=strcat('sorted_indices_control_control_lam_0p',num2str(10*lambda));
save(name,'ind_sort')

for i=1:p
    var_gen(i)=var(Supercomp_cancer_ref_control(:,i));
end

figure
stem(var_gen)

[var_sort ind_sort] = sort(var_gen,'descend');

name=strcat('sorted_variance_cancer_control_lam_0p',num2str(10*lambda));
save(name,'var_sort')
name=strcat('sorted_indices_cancer_control_lam_0p',num2str(10*lambda));
save(name,'ind_sort')
